#!/bin/bash

CONF_FILE='/rw2/etc/pbx/trunk_notification.conf'
CALLWEAVER='/usr/sbin/callweaver'
CALLWEAVER_PID='/var/run/callweaver/callweaver.pid'
IFS="#"

if [ ! -f "$CONF_FILE" ]; then
  cat << 'EOF' > $CONF_FILE
[general]
start=0
EOF
fi

[ "$(awk -F '=' '/start=/{print $2}' $CONF_FILE)" == "0" ] && exit
[ ! -f "$CALLWEAVER_PID" ] && exit
[ "$(($(date +%s) - $(stat -c %Y $CALLWEAVER_PID)))" -le "120" ] && exit


__FLAG() {

  local lock_file="/tmp/sip_trunk_$(md5sum <<< "$2" | awk '{print $1}').lock"
  local flag_status="$1"
  local try_count="2"

  [ -f "$lock_file" ] && local GET_TRY="$(cat $lock_file)"

  case "$flag_status" in
    UNSET)
      if [ "$GET_TRY" ]; then
        [ "$GET_TRY" -ge "$try_count" ] && echo "UP"
        rm -f $lock_file
      else
        rm -f $lock_file
      fi
    ;;
    SET)
      if [ "$GET_TRY" ]; then
        local SET_TRY="$(($GET_TRY+1))"
        [ "$SET_TRY" -le "10" ] && echo "$SET_TRY" > $lock_file
        [ "$SET_TRY" -eq "$try_count" ] && echo "DOWN"
      else
        echo "1" > $lock_file
      fi
    ;;
  esac

}


__SEND_MAIL() {

  if [ -s '/usr/sbin/ldapcmd' ]; then
    source /usr/sbin/ldapcmd
  else
    echo "ERROR: /usr/sbin/ldapcmd was not found" && return 1
  fi

  local sip_name="$1"
  local mail_type="$2"
  local mail_config='/etc/companies.d/0/msmtprc'
  local mail_from="$(awk '/mail_from/{print $2}' $mail_config)"
  local mail_admin="$(__LDAP_GET "(&(objectClass=wPerson)(userKind=admin)(cn=admin)) mail||mail" "dc=wildix" | awk '/mail/{print $2}')"
  local pbx_name="$(awk '/127.0.0.1/{print $4}' /etc/hosts)"

  # check mail password and admin mail
  [ ! "$mail_admin" ] || [ ! "$(awk '/password/{print $2}' $mail_config)" ] && { logger "CHECK_SIP: send email error (not set password or admin email)"; return 404; }

  # check mail_from
  [ ! "$mail_from" ] && mail_from="info@wildix.com"

  # set mail
  case "$mail_type" in
    DOWN) local SUBJECT_STATUS="Warning"      ; local SIP_STATUS="down" ;;
      UP) local SUBJECT_STATUS="Notification" ; local SIP_STATUS="up" ;;
  esac

  # send email
  echo "$sip_name" | sed 's/_/#/g' | while read HOST USERNAME NOT_USE; do
    echo -e "mail_from: $mail_from\nTo: $mail_admin\nSubject: $SUBJECT_STATUS $pbx_name: $HOST ($USERNAME) is $SIP_STATUS!\n\nsip: $HOST ($USERNAME)\ntime: $(date)\nstatus: $SIP_STATUS"|\
    msmtp -C $mail_config "$mail_admin"
  done

}


$CALLWEAVER -rx 'sip show registry' | awk '/Registered|Request Sent/{print $1"_"$3"_"$4"#"$5"#"$6"#"$7" "$8" "$9" "$10" "$11}'| sed 's/ws2sip.wildix.com/kite-connection/g'|\
while read SIP REFRESH STATUS LAST_REGISTRATION; do
    # check status sip registry
    case "$STATUS" in
      Registered) [ "$(__FLAG UNSET $SIP)" == "UP" ] && __SEND_MAIL "$SIP" "UP" ;;
         Request) [ "$(__FLAG SET $SIP)" == "DOWN" ] && __SEND_MAIL "$SIP" "DOWN" ;;
    esac
done
